import { ChartService } from '../../services/chart.service';
import defaultsDeep from 'lodash.defaultsdeep';
import { ChartDataService } from '../../services/chart-data.service';
export class Sparkline {
    constructor() {
        this.defaultConfig = {
            name: null,
            xAxis: {
                labelType: 'date',
                showAxis: false,
                showTicks: false
            },
            yAxis: {
                labelType: 'number',
                showAxis: false,
                showTicks: false
            }
        };
    }
    dataConfigHandler(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.chartSVG = this.create();
        }
    }
    componentDidLoad() {
        this.chartSVG = this.create();
    }
    async onLinesLoaded() {
        this.lines = await this.loadLines();
    }
    anjEndHoverHandler(svg) {
        window.clearTimeout(this.popoverTimer);
        this.activeIndex = null;
        this.popover.close();
        ChartService.hideFocus(svg);
    }
    anjHoverHandler(svg, dataConfig, x) {
        const dataIndex = ChartService.getIndexOfClosestPoint(x, dataConfig.data);
        let activeData;
        if (this.activeIndex !== dataIndex) {
            this.activeIndex = dataIndex;
            activeData = { dataConfig, activeIndex: this.activeIndex };
            this.anjHover.emit(activeData);
            this.popoverTimer = window.setTimeout(() => {
                const popoverEl = svg.select('.point-' + this.activeIndex).nodes()[0];
                this.display(popoverEl);
                ChartService.showFocus(svg, this.activeIndex);
            }, Sparkline.popoverDebounceLength);
        }
    }
    addEvents(svg, chartOverlay, x, dataConfig) {
        chartOverlay
            .on('mousemove', () => this.anjHoverHandler(svg, dataConfig, x))
            .on('mouseout', () => this.anjEndHoverHandler(svg));
    }
    appendSVG(svg) {
        while (this.chartContainer.children.length > 1) {
            this.chartContainer.removeChild(this.chartContainer.lastChild);
        }
        this.chartContainer.appendChild(svg.node());
    }
    async display(target) {
        const popper = await this.popover.popper();
        if (popper) {
            popper.reference = target;
            popper.scheduleUpdate();
        }
        else {
            this.popover.open(target);
        }
    }
    create() {
        const dataConfig = this.mergeDataConfig();
        const svg = ChartService.createSVG(dataConfig);
        const x = ChartService.addXAxis(svg, dataConfig);
        const y = ChartService.addYAxis(svg, dataConfig);
        if (this.lines) {
            this.hydrateLines(svg, dataConfig.data, x, y);
        }
        const chartOverlay = ChartService.addOverlay(svg, dataConfig);
        this.addEvents(svg, chartOverlay, x, dataConfig);
        ChartService.addRevealAnimation(svg, dataConfig);
        this.appendSVG(svg);
        return svg;
    }
    getDims() {
        const margin = { top: 20, right: 20, bottom: 20, left: 20 };
        const { height, width } = this.chartContainer.getBoundingClientRect();
        return {
            height: height - margin.top - margin.bottom,
            width: width - margin.left - margin.right,
            margin
        };
    }
    hydrateLines(svg, data, x, y) {
        this.lines.forEach((line, i) => {
            line.data = data;
            line.svg = svg;
            line.index = i;
            line.x = x;
            line.y = y;
            line.init();
        });
    }
    async loadLines() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-line')).map(col => col.componentOnReady()));
    }
    mergeDataConfig() {
        const config = defaultsDeep({}, this.dataConfig.config, this.defaultConfig, { dims: this.getDims() });
        const data = ChartDataService.formatData(config, this.dataConfig.data);
        return { config, data };
    }
    render() {
        return [
            h("anj-overlay", { className: "popover", placement: "top", offset: "0, 15", ref: r => this.popover = r },
                h("anj-arrow", null),
                h("slot", null))
        ];
    }
    static get is() { return "anj-sparkline"; }
    static get properties() { return {
        "chartContainer": {
            "elementRef": true
        },
        "dataConfig": {
            "type": "Any",
            "attr": "data-config",
            "mutable": true,
            "watchCallbacks": ["dataConfigHandler"]
        }
    }; }
    static get events() { return [{
            "name": "anjHover",
            "method": "anjHover",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "anjLineDidLoad",
            "method": "onLinesLoaded"
        }]; }
    static get style() { return "/**style-placeholder:anj-sparkline:**/"; }
}
Sparkline.popoverDebounceLength = 25;
